package TUI::Vision;

use strict;
use warnings;

our $VERSION = '2.000_001';
$VERSION =~ tr/_//d;
our $AUTHORITY = 'cpan:BRICKPOOL';

1;

__END__

=pod

=head1 NAME

TUI::Vision - Perl TUI Framework (Turbo Vision 2.0 Port)

=head1 SYNOPSIS

  use TUI::Vision;

  # This is a placeholder module.
  # The full implementation will be published in upcoming versions.

=head1 DESCRIPTION

TUI::Vision is a modern Perl Text User Interface (TUI) framework.
It is based on a clean, native port of Borland's Turbo Vision 2.0
(C++ 1994/97), adapted for contemporary Perl environments.

This module is currently just a placeholder. 

=head1 AUTHORS

=over

=item Borland International (original Turbo Vision design)

=item J. Schneider <brickpool@cpan.org> (Perl implementation and maintenance)

=back

=head1 CONTRIBUTORS

Contributors are documented in the POD of the respective framework modules.

=head1 COPYRIGHT AND LICENSE

Copyright (c) 1990-1994, 1997 by Borland International

Copyright (c) 2026 the L</AUTHORS> as listed above.

This software is licensed under the MIT license (see the LICENSE file, which is
part of the distribution).

=cut
